/* Unit level testbench for TSO code */

#include "tso_pre_include.h"
#include "tso_test_framework.h"
#include "../bitfield.h"
#include "../ef10_regs.h"
#define printf(...)

/* Ensure - all the IP packets end
 *        - all the data from the original SKB is seen
 */
int check_tx_queue(const struct efx_tx_queue *tx_queue,
		   struct tso_test_state *ts)
{
	bool lastcont = false, seen_option = false;
	int ip_payleft = -1; /* Count down so we know when the packet should end */
	unsigned int write_count = tx_queue->write_count;
	bool first_packet = true;
	u8 tcp_flags = 0;
	u16 ipv4_id = 0;
	u32 tcp_seqno = 0;

	/* Look through all the items on the queue */
	while (write_count != tx_queue->insert_count) {
		struct efx_tx_buffer *buf;
		unsigned int txq_pos;
		char *data;

		txq_pos = (write_count & tx_queue->ptr_mask);

		buf = &tx_queue->buffer[txq_pos];
		data = dma_addr_to_ptr(buf->dma_addr);

		if (ts->ops.hex_dump)
			ts->ops.hex_dump(data, buf->len);

		if (!lastcont) {
			/* deal with IP and TCP headers */
			
			unsigned int eth_hdr_len, ip_hdr_len, ip_paylen,
				tcp_hdr_len, tcp_paylen;
			struct iphdr *iph, *iph_copy;
			struct ipv6hdr *ip6h, *ip6h_copy;
			struct tcphdr *tcp;
			struct tcphdr *tcp_hdr_copy;
			__be16 protocol;

			if (buf->flags & EFX_TX_BUF_OPTION) {
				BUG_ON(tx_queue->efx->type->revision <
				       EFX_REV_HUNT_A0);
				BUG_ON(seen_option);
				BUG_ON(!EFX_QWORD_FIELD(
					       buf->option,
					       ESF_DZ_TX_DESC_IS_OPT));
				BUG_ON(EFX_QWORD_FIELD(
					       buf->option,
					       ESF_DZ_TX_OPTION_TYPE) !=
				       ESE_DZ_TX_OPTION_DESC_TSO);
				seen_option = true;
				tcp_flags = EFX_QWORD_FIELD(
					buf->option, ESF_DZ_TX_TSO_TCP_FLAGS);
				ipv4_id = EFX_QWORD_FIELD(
					buf->option, ESF_DZ_TX_TSO_IP_ID);
				tcp_seqno = EFX_QWORD_FIELD(
					buf->option, ESF_DZ_TX_TSO_TCP_SEQNO);
				goto next;
			}

			/* Assume that all headers are not fragmented.
			 * They are always linear in the skb, and they
			 * should never be long enough that we need to
			 * use multiple descriptors.
			 */
			BUG_ON(buf->len < ETH_HLEN);

			protocol = ((struct ethhdr *)data)->h_proto;
			if (protocol == htons(ETH_P_8021Q)) {
				eth_hdr_len = VLAN_ETH_HLEN;
				BUG_ON(buf->len < eth_hdr_len);
				protocol = (((struct vlan_ethhdr *)data)->
					    h_vlan_encapsulated_proto);
			} else {
				eth_hdr_len = ETH_HLEN;
			}
			ip_hdr_len = (protocol == htons(ETH_P_IP) ?
				      IP_HLEN : sizeof(struct ipv6hdr));
			BUG_ON(buf->len < eth_hdr_len + ip_hdr_len + TCP_HLEN);

			if (protocol == htons(ETH_P_IP)) {
				iph = (struct iphdr *)(data + eth_hdr_len);
				ip6h = NULL;
				iph_copy = (struct iphdr *)
					(ts->hdr_copy + eth_hdr_len);
				ip6h_copy = NULL;
			} else {
				iph = NULL;
				ip6h = (struct ipv6hdr *)(data + eth_hdr_len);
				iph_copy = NULL;
				ip6h_copy = (struct ipv6hdr *)
					(ts->hdr_copy + eth_hdr_len);
			}
			tcp = (struct tcphdr *)(data + eth_hdr_len + ip_hdr_len);
			tcp_hdr_copy = (struct tcphdr *)(ts->hdr_copy + eth_hdr_len + ip_hdr_len);
			tcp_hdr_len = tcp_hdr_copy->doff * 4;

			BUG_ON(buf->len < eth_hdr_len + ip_hdr_len + tcp_hdr_len);

			if (seen_option) {
				/* Firmware will fix up lengths
				 * automatically, and other fields as
				 * specified in the option descriptor.
				 *
				 * XXX We really shouldn't modify the
				 * TX buffer in-place.
				 */
				tcp_paylen =
					min(ts->tcp_paylen_left, ts->gso_size);
				ip_paylen = tcp_hdr_len + tcp_paylen;
				if (iph) {
					iph->id = htons(ipv4_id);
					iph->tot_len = htons(ip_hdr_len +
							     tcp_hdr_len +
							     tcp_paylen);
				} else {
					ip6h->payload_len = htons(tcp_hdr_len +
								  tcp_paylen);
				}
				tcp->seq = htonl(tcp_seqno);
				((u8 *)tcp)[13] = tcp_flags;
			} else {
				if (iph)
					ip_paylen = ntohs(iph->tot_len) -
						ip_hdr_len;
				else
					ip_paylen = ntohs(ip6h->payload_len);
				tcp_paylen = ip_paylen - tcp_hdr_len;
				BUG_ON(tcp_paylen !=
				       min(ts->tcp_paylen_left, ts->gso_size));
			}

			/* Sort out the expected flags */
			ts->tcp_paylen_left -= tcp_paylen;

			printf("Header: IP=%-6d+%-6d=%6d    TCP=%-6d+%-6d=%-6d    "
			       "tcp_paylen_left=%-6d\n",
			       ip_hdr_len, ip_paylen, ip_hdr_len + ip_paylen,
			       ip_paylen-tcp_paylen, tcp_paylen, ip_paylen,
			       ts->tcp_paylen_left);

			/* Check the headers */
			if (ts->ops.check_buffer != NULL) {

				/* Adjust the expected headers */
				/* TODO: Could do this check without globals */
				if (ts->tcp_paylen_left == 0) {
					tcp_hdr_copy->fin = ts->fin;
					tcp_hdr_copy->psh = ts->psh;
				} else {
					tcp_hdr_copy->fin = 0;
					tcp_hdr_copy->psh = 0;
				}
				if (first_packet)
					tcp_hdr_copy->cwr = ts->cwr;
				else
					tcp_hdr_copy->cwr = 0;
				first_packet = false;

				if (iph_copy)
					iph_copy->tot_len =
						htons(ip_hdr_len + tcp_hdr_len +
						      tcp_paylen);
				else
					ip6h_copy->payload_len =
						htons(tcp_hdr_len + tcp_paylen);

				if (ts->ops.check_buffer(buf, ts->hdr_copy, BUF_HEADER) != 0)
					return -1;

				/* Consume some data space if the header had data */
				ts->data_to_check =
					ts->data_to_check + buf->len - eth_hdr_len -
					ip_hdr_len - tcp_hdr_len;

				/* Adjust the next expected sequence number
				 * and IPv4 ID
				 */
				/* TODO: check this when not cheking data */
				tcp_hdr_copy->seq = htonl(ntohl(tcp_hdr_copy->seq) + tcp_paylen);
				if (iph_copy)
					iph_copy->id =
						htons(ntohs(iph_copy->id) + 1);
			}

			/* Account for how much IP data we need to see to complete the header 
			 */
			ip_payleft = ip_paylen - (buf->len - eth_hdr_len - ip_hdr_len);

			/* ip_paylen used instead of ip_payleft as not all of this frag is ip_payload */
			printf("TXQ[%4.4d]:   @0x%p  len=%-6d  ip_payleft %-6d "
			       "cont=%d\n", txq_pos, dma_addr_to_ptr(buf->dma_addr),
			       buf->len, ip_paylen, buf->flags & EFX_TX_BUF_CONT);


		} else {
			BUG_ON(buf->flags & EFX_TX_BUF_OPTION);

			printf("TXQ[%4.4d]:   @0x%p  len=%-6d  ip_payleft %-6d "
			       "cont=%d\n", txq_pos, dma_addr_to_ptr(buf->dma_addr),
			       buf->len, ip_payleft, buf->flags & EFX_TX_BUF_CONT);

			/* Check the data */
			if (ts->ops.check_buffer != NULL) {
				if (ts->ops.check_buffer(buf, ts->data_to_check, BUF_DATA) != 0)
					return -1;
				ts->data_to_check = ts->data_to_check + buf->len;
			}
			ip_payleft -= buf->len;
		}

		BUG_ON(ip_payleft < 0);
		BUG_ON(ts->tcp_paylen_left < 0);
		lastcont = (ip_payleft == 0) ? 0 : 1;
		BUG_ON(lastcont != (buf->flags & EFX_TX_BUF_CONT));
		if (!lastcont)
			seen_option = false;

		/* Prepare for next loop */
	next:
		txq_pos = (txq_pos + 1) & (tx_queue->ptr_mask);
		write_count++;
	}
	/* Check all headers resulted in all the data following on */
	BUG_ON(ip_payleft != 0);
	/* Check that there is not a dangling CONT bit or option */
	BUG_ON(lastcont);
	BUG_ON(seen_option);
	/* Check that everything from the SKB was sent */
	BUG_ON(ts->tcp_paylen_left != 0);
	return 0;
}

/*
 * Local variables:
 *  c-basic-offset: 8
 *  c-indent-level: 8
 *  tab-width: 8
 *  indent-tabs-mode: 1
 * End:
 */
