#include "tso_pre_include.h"
#include "tso_test_framework.h"
#include "net_driver.h"

#define MY_DMA_ALLOC_MAGIC 0xefab0d3a

struct my_dma_alloc {
	u32 magic;
	void *base;
	size_t size;
};

void *my_dma_alloc_coherent(struct device *hwdev, size_t size, dma_addr_t *dma_handle)
{
	struct my_dma_alloc *alloc;
	void *base, *aligned;

	base = my_malloc(sizeof(struct my_dma_alloc) + size + PAGE_SIZE);
	aligned = EFX_PTR_ALIGN_FWD(base + sizeof(struct my_dma_alloc),
				    PAGE_SIZE);
	alloc = (struct my_dma_alloc *)aligned - 1;
	alloc->magic = MY_DMA_ALLOC_MAGIC;
	alloc->base = base;
	alloc->size = size;

	*dma_handle = ptr_to_dma_addr(aligned);

	return aligned;
}

void my_dma_free_coherent(struct device *hwdev, size_t size, void *vaddr, dma_addr_t dma_handle)
{
	struct my_dma_alloc *alloc;

	if (vaddr == NULL)
		return;

	BUG_ON(dma_handle != ptr_to_dma_addr(vaddr));
	alloc = (struct my_dma_alloc *)vaddr - 1;
	BUG_ON(alloc->magic != MY_DMA_ALLOC_MAGIC);
	BUG_ON(alloc->size != size);
	my_free(alloc->base);
}

dma_addr_t my_dma_map_page(struct device *hwdev, struct page *page, size_t offset, size_t size, int direction)
{
	return ptr_to_dma_addr(page_address(page)) + offset;
}

void my_dma_unmap_page(struct device *hwdev, dma_addr_t dma_addr, size_t size, int direction)
{
}

dma_addr_t my_dma_map_single(struct device *hwdev, void *ptr, size_t size, int direction)
{
	return ptr_to_dma_addr(ptr);
}

void my_dma_unmap_single(struct device *hwdev, dma_addr_t dma_addr, size_t size, int direction)
{
}

int my_dma_mapping_error(struct device *hwdev, dma_addr_t dma_addr)
{
	return 0;
}

int my_dev_kfree_skb_any(struct sk_buff *skb)
{
	/* Should never be called, as there are no TX completions */
	BUG_ON(1);
	return 0;
}

#if !defined(EFX_USE_KCOMPAT) || !defined(EFX_NEED_CONSUME_SKB_ANY)
int my_dev_consume_skb_any(struct sk_buff *skb)
{
	/* Should never be called, as we don't expect any TSO fallbacks */
	BUG_ON(1);
	return 0;
}
#endif

void *my_page_address(struct page *page)
{
	return (void *)page->private;
}

int my_skb_copy_bits(const struct sk_buff *skb, int offset, void *to, int len)
{
	const struct skb_frag_struct *frag = skb_shinfo(skb)->frags;
	int copy;

	if (offset > (int)skb->len - len)
		return -EFAULT;

	if (offset < skb_headlen(skb)) {
		copy = min_t(int, len, skb_headlen(skb) - offset);
		memcpy(to, skb->data + offset, copy);
		to += copy;
		len -= copy;
		offset = 0;
	} else {
		offset -= skb_headlen(skb);
	}

	while (len) {
		if (offset < skb_frag_size(frag)) {
			copy = min_t(int, len, skb_frag_size(frag) - offset);
			memcpy(to, skb_frag_address(frag), copy);
			to += copy;
			len -= copy;
			offset = 0;
		} else {
			offset -= skb_frag_size(frag);
		}
		++frag;
	}

	return 0;
}

int my_skb_cow_head(struct sk_buff *skb, unsigned int headroom)
{
	return 0;
}

int my_pskb_expand_head(struct sk_buff *skb, int nhead, int ntail, gfp_t gfp_mask)
{
	return 0;
}

void my_warn_slowpath_null(const char *file, int line)
{
	my_fail("WARN at %s:%d\n", file, line);
}

unsigned long jiffies;

int my_efx_nic_alloc_buffer(struct efx_nic *efx, struct efx_buffer *buffer,
			    unsigned int len, gfp_t flags)
{
	buffer->addr = my_dma_alloc_coherent(NULL, len, &buffer->dma_addr);
	if (!buffer->addr)
		return -ENOMEM;
	buffer->len = len;
	return 0;
}

void my_efx_nic_free_buffer(struct efx_nic *efx, struct efx_buffer *buffer)
{
	my_dma_free_coherent(NULL, buffer->len, buffer->addr, buffer->dma_addr);
}

void my_efx_schedule_reset(struct efx_nic *efx, enum reset_type type)
{
	my_fail("efx_schedule_reset called\n");
}

struct kernel_param;
#if !defined(EFX_USE_KCOMPAT) || !defined(EFX_HAVE_NON_CONST_KERNEL_PARAM)
int param_get_uint(char *buffer, const struct kernel_param *kp)
#else
int param_get_uint(char *buffer, struct kernel_param *kp)
#endif
{
	my_fail("param_get_uint called?!\n");
	return 0;
}
#if !defined(EFX_USE_KCOMPAT) || !defined(EFX_HAVE_NON_CONST_KERNEL_PARAM)
int param_set_uint(const char *buffer, const struct kernel_param *kp)
#else
int param_set_uint(const char *buffer, struct kernel_param *kp)
#endif
{
	my_fail("param_set_uint called?!\n");
	return 0;
}

#if !defined(EFX_USE_KCOMPAT) || defined(EFX_HAVE_VOID_DYNAMIC_NETDEV_DBG)
void __dynamic_netdev_dbg(struct _ddebug *descriptor,
			 const struct net_device *dev,
			 const char *fmt, ...)
{
	return;
}
#else
int __dynamic_netdev_dbg(struct _ddebug *descriptor,
			 const struct net_device *dev,
			 const char *fmt, ...)
{
	return 0;
}
#endif

void my_memcpy_toio(volatile void *to, const void *from, unsigned int len)
{
	/* TSO should never use PIO */
	BUG_ON(1);
}

void my_page_frag_free(void *p)
{
	/* Only ever used for XDP TX completions */
	BUG_ON(1);
}
