/* Unit level testbench for TSO code
 *  - constructs an skb, passes it through TSO and then checks the tx_queue
 *    for the correct data
 * 
 * TODO: TXQ becoming full
 *       Wrapping around the tx_queue boundary
 *       random ip options and tcp options
 *       Check no fragments go over 512 byte limits
 *       Interaction between TSO and none-TSO data
 *       Mix with non-TCP random packets
 *       More intersting fragmentation
 *       Use a CRC checking routine that could go into tso.c
 */

/* Notes: skb->gso_size is MSS
 */

#include "tso_pre_include.h"
#include "tso_test_framework.h"

#define MAX(x,y) ((x) > (y) ? (x) : (y))


/* settings */
#define GSO_MAX_SEG_SIZE 9000

#define FRAG_MAX_SIZE 9000
#define FRAG_SMALL_SIZE 16
#define FRAG_SMALL_PC 25
#define FRAG_MAX 6

#define RAND_DATA_SIZE (FRAG_MAX_SIZE * FRAG_MAX)

struct tso_test_state ts;

/***** OVERRIDES **************************************************************/
int ___pskb_trim(struct sk_buff *skb, unsigned int len)
{
	MY_ASSERT(skb->len <= len);
	return 0;
}

/***** HELPER ROUTINES ********************************************************/

#define rand my_rand
static unsigned int my_rand(unsigned long seed)
{
	static unsigned long i;

	if (unlikely(seed)) {
		i = seed;
		return i;
	}

	/* From http://www.ece.cmu.edu/~koopman/lfsr/ */
	if (i & 1)
		i = (i >> 1) ^ 0x80000B84;
	else
		i = (i >> 1);

	return i;
}


static unsigned int rand_halfopen(unsigned int min, unsigned int max)
{
	unsigned int range = max - min;

	MY_ASSERT(min < max);
	return min + (rand(0) % range);
}

static unsigned int rand_closed(unsigned int min, unsigned int max)
{
	return rand_halfopen(min, max + 1);
}


/* a hexdump that is compatible with pktdump */
static void hex_dump(const uint8_t * data, unsigned int len)
{
	unsigned int i;
	char *bufptr, *bufbase;
	bufptr = bufbase = my_malloc(16 * 3 + 1);

	if (!data) {
		printk("hex_dump() called with NULL data\n");
		return;
	}

	if (len == 0) {
		printk("hex_dump() called with no data\n");
		return;
	}

	for (i = 0; i < len; i++) {
		bufptr += snprintf(bufptr, 4, "%02x", data[i]);
		if ((i & 1) == 1)
			bufptr += snprintf(bufptr, 2, " ");
		if ((i & 3) == 3)
			bufptr += snprintf(bufptr, 3, "  ");
		if ((i % 0x10) == 0x0f) {
			if (verbose)
				my_printf("%08x  %s\n", (i & ~0x0f), bufbase);
			bufptr = bufbase;
		}
	}
	if (bufptr != bufbase)
		if (verbose)
			my_printf("%08x  %s\n", (i & ~0x0f), bufbase);
	my_free(bufbase);
}


static void dump_tx_queue(struct efx_tx_queue *tx_queue)
{
	unsigned int write_count = tx_queue->write_count;

	while (write_count != tx_queue->insert_count) {
		unsigned int txq_pos;
		struct efx_tx_buffer *buf;

		txq_pos = (write_count & tx_queue->ptr_mask);
		buf = &tx_queue->buffer[txq_pos];

		my_printf("TXQ[%d] len %d addr 0x%llx cont=%d\n",
		       write_count, buf->len, (u64)buf->dma_addr,
		       buf->flags & EFX_TX_BUF_CONT);

		write_count++;
	}
}


static void dump_skb(struct sk_buff *skb)
{
	unsigned int i, tot_len;
	unsigned int ip_hdr_len, tcp_hdr_len, ip_payload, tcp_payload;

	if (skb->protocol == htons(ETH_P_IP)) {
		ip_hdr_len = ip_hdr(skb)->ihl * 4;
		tot_len = ntohs(ip_hdr(skb)->tot_len);
	} else {
		ip_hdr_len = sizeof(struct ipv6hdr);
		tot_len = ip_hdr_len + ntohs(ipv6_hdr(skb)->payload_len);
	}
	tcp_hdr_len = tcp_hdr(skb)->doff * 4;
	ip_payload = tot_len - ip_hdr_len;
	tcp_payload = ip_payload - tcp_hdr_len;


	my_printf("SKB @0x%p is  |%s  IPv%d %5d+%5d=%5d  |  TCP %5d+%5d=%5d tcp_paylen_left=%5d  |  ",
	       skb, skb_vlan_tag_present(skb) ? "  VLAN  |" : "",
	       ip_hdr(skb)->version, ip_hdr_len, ip_payload, tot_len,
	       tcp_hdr_len, tcp_payload, ip_payload,
	       ts.tcp_paylen_left);

	my_printf("%s ", tcp_hdr(skb)->psh ? "PSH" : "   ");
	my_printf("%s ", tcp_hdr(skb)->rst ? "RST" : "   ");
	my_printf("%s ", tcp_hdr(skb)->urg ? "URG" : "   ");
	my_printf("%s ", tcp_hdr(skb)->cwr ? "CWR" : "   ");

	/* print lengths */
	my_printf("[ %d ", skb_headlen(skb));
	for (i = 0; i < skb_shinfo(skb)->nr_frags; i++)
		my_printf("%d ", skb_shinfo(skb)->frags[i].size);

	/* print page offsets */
	my_printf("] [");
	for (i = 0; i < skb_shinfo(skb)->nr_frags; i++)
		my_printf("0x%x ", skb_shinfo(skb)->frags[i].page_offset);
	my_printf("]\n");

	MY_ASSERT(ip_payload >= 20);
	MY_ASSERT(tcp_payload >= 0);
}


/* Allocate memory but use an alignment set by the random seed */
static void *malloc_rand_aligned(void **free_addr, unsigned int size)
{
	char *addr = my_malloc(size + PAGE_SIZE * 4);
	if (!addr)
		my_fail("Out of memory");

	*free_addr = addr;
	addr = EFX_PTR_ALIGN_FWD(addr, PAGE_SIZE * 2);
	addr += rand_halfopen(0, PAGE_SIZE * 2);

	return addr;
}

/***** SETUP/FINALISATION *****************************************************/

static void tx_write(struct efx_tx_queue *tx_queue)
{
}

static unsigned int tx_limit_len(struct efx_tx_queue *tx_queue,
				 dma_addr_t dma_addr, unsigned int len)
{
	return len;
}

static int do_test_init(struct efx_channel *channel, int nic_revision)
{
	struct efx_nic *nic;
	struct efx_nic_type *nic_type;
	struct efx_tx_queue *tx_queue;
	const unsigned int txqsize = 1024; // XXX rjs TODO
	int i;

	nic = (struct efx_nic *)my_calloc(1, sizeof(struct efx_nic));
	if (!nic)
		return -ENOMEM;

	nic_type = (struct efx_nic_type *)my_calloc(1, sizeof(*nic_type));
	if (!nic_type)
		return -ENOMEM;

	channel->tx_queues = (struct efx_tx_queue *) my_calloc(EFX_TXQ_TYPES,
							sizeof(struct efx_tx_queue));
	if (!channel->tx_queues)
		return -ENOMEM;

	/* Setup the txq */
	nic_type->revision = nic_revision;
	nic_type->tx_write = tx_write;
	nic_type->tx_limit_len = tx_limit_len;
	nic->txq_entries = txqsize;
	nic->type = nic_type;
	channel->efx = nic;
	for (i = 0; i < EFX_TXQ_TYPES; i++) {
		tx_queue = &channel->tx_queues[i];
		tx_queue->efx = nic;
		tx_queue->queue = i;
		tx_queue->channel = channel;
		efx_probe_tx_queue(tx_queue);
		efx_init_tx_queue(tx_queue);
	}
	tx_queue = &channel->tx_queues[EFX_TXQ_TYPE_CSUM_OFFLOAD];

	for (i = 0; i < txqsize; i++) {
		/* Only for some asserts */
		tx_queue->buffer[i].flags = 0;
		tx_queue->buffer[i].len = 0;
		tx_queue->buffer[i].unmap_len = 0;
	}

	/* Allocate the data buffer */
	ts.data_base = malloc_rand_aligned(&ts.data_base_free, RAND_DATA_SIZE);
	ts.data_left = RAND_DATA_SIZE;

	ts.data_to_send = ts.data_base;
	ts.data_to_check = ts.data_base;

	if (!ts.data_base)
		return -ENOMEM;

	/* Setup an interesting data pattern - OK it isn't interesting */
	for (i = 0; i < ts.data_left; i++)
		ts.data_base[i] = i % 256; //TODO

	return 0;
}


static void do_test_fini(struct efx_channel *channel)
{
	struct efx_tx_queue *tx_queue;
	int i;

	for (i = 0; i < 2; i++) {
		tx_queue = &channel->tx_queues[i];
		efx_fini_tx_queue(tx_queue);
		efx_remove_tx_queue(tx_queue);
		efx_destroy_tx_queue(tx_queue);
	}
	my_free(channel->tx_queues);
	my_free((void *)channel->efx->type);
	my_free(channel->efx);

	my_free(ts.data_base_free);
	ts.data_base = NULL;

	my_free(ts.hdr_copy);
}


static int test_alloc_skb(struct efx_tx_queue *tx_queue,
			  unsigned int tcp_pay_in_skb, struct sk_buff **out_skb)
{
	struct sk_buff *skb;
	unsigned int ip_hdr_len, tot_len, i, nr_frags, size;
	__be16 protocol;
	unsigned int min_gso, max_gso;

	if (rand_halfopen(0, 2)) {
		protocol = htons(ETH_P_IP);
		ip_hdr_len = IP_HLEN;
	} else {
		protocol = htons(ETH_P_IPV6);
		ip_hdr_len = sizeof(struct ipv6hdr);
	}

	skb = (struct sk_buff *)my_malloc(sizeof(struct sk_buff));
	if (!skb)
		return -ENOMEM;

	//XXX rjs
	nr_frags = rand_closed(1, FRAG_MAX);

	/* Ensure there is some TCP payload */
	if (nr_frags == 0)
		tcp_pay_in_skb = (tcp_pay_in_skb < 1) ? 1 : tcp_pay_in_skb;
	size = ETH_HLEN + ip_hdr_len + TCP_HLEN + tcp_pay_in_skb; //XXX rjs

	skb->truesize = size + sizeof(struct skb_shared_info);
	skb->head = (char *)malloc_rand_aligned(&ts.skb_head_free, skb->truesize);
	if (!skb->head)
		return -ENOMEM;

	/* Setup skb */
	skb->data = skb->head; /* to allow headroom */
	skb_reset_tail_pointer(skb); /* to allow tailroom */
#ifdef NET_SKBUFF_DATA_USES_OFFSET
	skb->end = size;
#else
	skb->end = skb->head + size;
#endif

	skb->len = size;
	skb->data_len = 0;

	/* TODO: Add random data into the header */
	{
		unsigned int hdr_size = size - tcp_pay_in_skb;

		/* This is the header */
		for (i = 0; i < size - tcp_pay_in_skb; i++) {
			char rand_data = rand_halfopen(0, 256);
			*(skb->data + i) = clean_headers ? 0 : rand_data;
		}

		/* Extra data within the SKB */
		ts.data_to_send += tcp_pay_in_skb;
		for (i = 0; i < tcp_pay_in_skb; i++)
			*(skb->data + hdr_size + i) = i % 256;
	}

	/* Setup the packet  - eth */
	skb->protocol = protocol;
	((struct ethhdr *)skb->data)->h_proto = protocol;
	if (rand_halfopen(0, 2))
		__vlan_hwaccel_put_tag(skb, htons(ETH_P_8021Q), rand_halfopen(0, 0x10000));
	else
		skb->vlan_tci = 0;

	/* setup - ip */
	skb_set_network_header(skb, ETH_HLEN);
	if (protocol == htons(ETH_P_IP)) {
		ip_hdr(skb)->protocol = IPPROTO_TCP;
		ip_hdr(skb)->version = 4;
		ip_hdr(skb)->ihl = ip_hdr_len >> 2;
	} else {
		ipv6_hdr(skb)->nexthdr = NEXTHDR_TCP;
		ipv6_hdr(skb)->version = 6;
	}
	tot_len = size - ETH_HLEN;

	/* setup - tcp */
	skb_set_transport_header(skb, ETH_HLEN + ip_hdr_len);
	tcp_hdr(skb)->doff = 20 >> 2;
	if (tcp_no_urg)
		tcp_hdr(skb)->urg = 0;
	if (tcp_no_rst)
		tcp_hdr(skb)->rst = 0;
	if (tcp_no_syn)
		tcp_hdr(skb)->syn = 0;

	ts.tcp_paylen_left = (skb_headlen(skb) - ETH_HLEN - ip_hdr_len -
			      tcp_hdr(skb)->doff * 4);

	/* allocate fragments */
	for (i = 0; i < nr_frags; i++) {
		struct page *page;
		unsigned int smallf, fsize, fmaxsize;

		smallf = rand_halfopen(0, 100) < FRAG_SMALL_PC;
		fmaxsize = smallf ? FRAG_SMALL_SIZE : FRAG_MAX_SIZE;
		fsize = rand_closed(1, fmaxsize);

		/* remove pathological case if 1 fragment and 1 byte of data */
		if (nr_frags == 1 && fsize == 1)
			fsize = 2;

		page = my_calloc(1, sizeof(struct page));
		if (!page)
			return -ENOMEM;

		page->private = (unsigned long)ts.data_to_send & PAGE_MASK;
		skb_fill_page_desc(skb, i, page,
				   (unsigned long)ts.data_to_send &
				   ~PAGE_MASK,
				   fsize);

		ts.data_to_send += fsize;
		ts.data_left -= fsize;

		skb->data_len += fsize;
		skb->len += fsize;
		tot_len += fsize;
		ts.tcp_paylen_left += fsize;
	}

	if (skb_vlan_tag_present(skb)) {
		struct vlan_ethhdr *veh;
		ts.hdr_copy = (char *)my_malloc(skb_headlen(skb) + VLAN_HLEN);
		veh = (struct vlan_ethhdr *)ts.hdr_copy;
		memcpy(ts.hdr_copy, skb->data, 2 * ETH_ALEN);
		veh->h_vlan_proto = htons(ETH_P_8021Q);
		veh->h_vlan_TCI = htons(skb_vlan_tag_get(skb));
		memcpy(ts.hdr_copy + 2 * ETH_ALEN + VLAN_HLEN,
		       skb->data + 2 * ETH_ALEN,
		       skb_headlen(skb) - 2 * ETH_ALEN);
	} else {
		ts.hdr_copy = (char *)my_malloc(skb_headlen(skb) + VLAN_HLEN);
		memcpy(ts.hdr_copy, skb->data, skb_headlen(skb));
	}

	/* calculate expected flags */
	ts.fin = tcp_hdr(skb)->fin;
	ts.psh = tcp_hdr(skb)->psh;
	ts.cwr = tcp_hdr(skb)->cwr;

	/* fix total length */
	if (protocol == htons(ETH_P_IP))
		ip_hdr(skb)->tot_len = htons(tot_len);
	else
		ipv6_hdr(skb)->payload_len = htons(tot_len - ip_hdr_len);

	/* print info out */
	skb_shinfo(skb)->nr_frags = nr_frags;
	if (verbose)
		dump_skb(skb);

	BUG_ON(!ts.tcp_paylen_left);

	/* pick random gso_size - this is TCP MSS */

	/* must be split into two packets */
	max_gso = min(ts.tcp_paylen_left - 1, GSO_MAX_SEG_SIZE);
 
	/* must not exceed the maximum supported number of segments */
	min_gso = DIV_ROUND_UP(ts.tcp_paylen_left, EFX_TSO_MAX_SEGS);
 
	ts.gso_size = rand_closed(min_gso, max_gso);
	skb_shinfo(skb)->gso_size = ts.gso_size;
	skb_shinfo(skb)->gso_segs = DIV_ROUND_UP(ts.tcp_paylen_left, ts.gso_size);

	*out_skb = skb;
	return 0;
}


static void test_free_skb(struct sk_buff *skb)
{
	unsigned int i;

	for (i = 0; i < skb_shinfo(skb)->nr_frags; ++i)
		my_free(skb_frag_page(&skb_shinfo(skb)->frags[i]));
	my_free(ts.skb_head_free);
	my_free(skb);
}


static void test_fix_counts(struct efx_tx_queue *tx_queue)
{
	tx_queue->write_count = tx_queue->insert_count;
	tx_queue->read_count = tx_queue->insert_count;
}

/**** CHECKING ROUTINES *****************************************************/

#define BUF_HEADER 1
#define BUF_DATA 0
int check_buffer(struct efx_tx_buffer *txbuf, const char *expect, int header)
{
	char *type;
	char *data = (char *) dma_addr_to_ptr(txbuf->dma_addr);
	type = header ? "header" : "data";

	if (memcmp(expect, data, txbuf->len) != 0) {
		my_printf("Error in %s comparison. Got:\n", type);
		hex_dump(data, txbuf->len);
		my_printf("Expected:\n");
		hex_dump(expect, txbuf->len);
		return -1;
	}
	return 0;
}


/**** MAIN ******************************************************************/
int do_test(void)
{
	int rc;
	struct sk_buff *skb;
	struct efx_channel *channel;
	struct efx_tx_queue *tx_queue;
	unsigned int i;

	if (iters == 0) {
		my_printf("0 iterations specified\n");
		return 2;
	}

	if (nic_revision < EFX_REV_SIENA_A0 ||
	    nic_revision > EFX_REV_HUNT_A0) {
		my_printf("invalid nic-revision\n");
		return 2;
	}

	seed = rand(seed);
	ts.ops.hex_dump = do_hex_dump ? hex_dump : NULL;
	ts.ops.check_buffer = check_buffer;

	for (i = 0; i < iters; i++) {
		unsigned int tcp_pay_in_skb = 0;

		if (verbose)
			my_printf("===== seed=%lu =======================================================\n",
				  seed);

		channel = my_calloc(1, sizeof(*channel));
		if (!channel)
			return -ENOMEM;

		rc = do_test_init(channel, nic_revision);
		if (rc != 0)
			return rc;

		tx_queue = &channel->tx_queues[EFX_TXQ_TYPE_CSUM_OFFLOAD];

		if (tcp_rand_pay_in_skb)
			tcp_pay_in_skb = rand_halfopen(0, 64);

		if ((rc = test_alloc_skb(tx_queue, tcp_pay_in_skb, &skb)) != 0)
			return rc;
		if (!skb)
			return -ENOMEM;

		if (verbose)
			my_printf("==================== Test iteration %d gso_size=%d tcp_pay_in_skb=%d ====================\n",
				  i, ts.gso_size, tcp_pay_in_skb);

		/* Also frees the skb */
		if (efx_enqueue_skb(tx_queue, skb) != NETDEV_TX_OK)
			return -1;

		MY_ASSERT(tx_queue->insert_count > 0);

		if (do_dump_tx_queue)
			dump_tx_queue(tx_queue);
		if (check_tx_queue(tx_queue, &ts) != 0)
			return -2;

		/* clean up */
		test_free_skb(skb);
		test_fix_counts(tx_queue);
		do_test_fini(channel);
		my_free(channel);

		/* Setup for next loop */
		seed = rand(0);
	}

	return 0;
}

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>

MODULE_LICENSE("Proprietary");

/*
 * Local variables:
 *  c-basic-offset: 8
 *  c-indent-level: 8
 *  tab-width: 8
 *  indent-tabs-mode: 1
 * End:
 */
