#!/bin/bash
# SPDX-License-Identifier: GPL-2.0
#
# Multidevice: Using pktgen threads for sending on multiple devices:
#  * Takes a comma-separaed list of devices
#  * Adds devices to kernel threads.
#  * Flow variation via random UDP source port.
#  * Support for xmit-more (called bursting).
#  * Use CPU 0 for this script, and from CPU 1 onwards for pktgen threads.
#  * Accept a target line rate
#
# Call this without arguments to get help.
#
# Adapted from linux/samples/pktgen/pktgen_sample02_multiqueue.sh
# commit a2b7ab45b890.
#
basedir=`dirname $0`
source ${basedir}/functions.sh
root_check_run_with_sudo "$@"
#
# Required param: -i dev in $DEV
source ${basedir}/parameters.sh

# Default params
if [ -z "$DEST_IP" ]; then
    [ -z "$IP6" ]   && DEST_IP="198.18.0.42" || DEST_IP="FD00::1"
fi
[ -z "$DST_MAC" ]   && DST_MAC="90:e2:ba:ff:ff:ff"
[ -z "$BURST" ]     && BURST=32
[ -z "$COUNT" ]     && COUNT="0" # Zero means indefinitely
[ -z "$CLONE_SKB" ] && CLONE_SKB="1024"

# Base Config
DELAY="0"        # Zero means max speed

# Flow variation random source port between min and max
UDP_MIN=9
UDP_MAX=109

# General cleanup everything since last run
pg_ctrl "reset"

# Bind this script to CPU 0.
taskset -p 1 $$ > /dev/null
# Start pktgen from CPU 1 onwards.
thread=1

# Interfaces are specified with parameter -i value in $DEV
for dev in ${DEV//,/ }
do
    # Add remove all other devices and add_device $dev to thread
    pg_thread $thread "rem_device_all"
    pg_thread $thread "add_device" $dev

    # Notice config queue to map to cpu (mirrors smp_processor_id())
    # It is beneficial to map IRQ /proc/irq/*/smp_affinity 1:1 to CPU number
    pg_set $dev "flag QUEUE_MAP_CPU"

    # Base config of dev
    pg_set $dev "count $COUNT"
    pg_set $dev "clone_skb $CLONE_SKB"
    pg_set $dev "pkt_size $PKT_SIZE"
    pg_set $dev "delay $DELAY"

    # Flag example disabling timestamping
    pg_set $dev "flag NO_TIMESTAMP" 2> /dev/null

    # Destination
    pg_set $dev "dst_mac $DST_MAC"
    pg_set $dev "dst$IP6 $DEST_IP"

    # Setup random UDP port src range
    pg_set $dev "flag UDPSRC_RND"
    pg_set $dev "udp_src_min $UDP_MIN"
    pg_set $dev "udp_src_max $UDP_MAX"

    # Setup burst, for easy testing -b 0 disable bursting
    # (internally in pktgen default and minimum burst=1)
    have_bursting=`grep burst /proc/net/pktgen/$dev`
    test -z "$have_bursting" && BURST=0

    if [[ ${BURST} -ne 0 ]]; then
        pg_set $dev "burst $BURST"
    else
        echo "$dev: Not using burst"
    fi

    test -n "$RATE" && pg_set $dev "rate $RATE"

    let thread=$thread+1
done

stop_run() {
	test -z "$VERBOSE" && exit 0
	echo "$0 Done"

	# Print results
	for dev in ${DEV//,/ }
	do
	    echo "Device: $dev"
	    cat /proc/net/pktgen/$dev | grep -A2 "Result:"
	done
}

# start_run
test -n "$VERBOSE" && echo "Running... ctrl^C to stop"
trap stop_run SIGINT SIGTERM
pg_ctrl "start"
